package Apache::Grapher;
# File: Apache/Grapher.pm

# Grapher.pm (c) 2000 Chris Snell <chris@bikeworld.com>
# Grapher.pm is a mod_perl CGI that uses arguments (usually
# passed to it by 'chef') to product RRD graphs in PNG format.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


use strict;
use Apache;
use Apache::Constants qw( :common :response );
use FileHandle;
use URI::Escape;
use Time::HiRes qw(gettimeofday);

# If you want to use ImageMagick to make on-the-fly
# thumbnails, you will need to uncomment the following
# line and change the '0' in the next line to a '1'.
use Image::Magick;
my $have_im = 1; # Set to '1' to enable ImageMagick Support

# The Shared RRDtool module
use RRDs;

my $RRD_BASE = "/etc/bronc/rrd_files";

############################################
# INPUTS TAKEN........                     #
############################################
# t#  - target name                        #
# fi# - RRD filename containing target     # 
# tl# - target label in legend             # 
# c#  - color of target                    #
# gt# - plot type for target               #
# ti  - title at top of graph              #
# vl  - y-axis (vertical) label            #
# gw  - graph area width (pixels)          #
# gh  - graph area height (pixels)         #
# il  - interlaced (yes/no)                #
# lg  - logarithmic y-axis scale (yes/no)  #
# st  - start time                         #
# et  - end time                           #
# lt# - legend target                      #
# lf# - format                             #
# lx# - legend text                        #
# cp# - composite target definition        #
# ct# - composite def. title               #
# sx  - resize graph width to # pixels     #
# sy  - resize graph height to # pixels    #
############################################


sub handler {
	# get request object
	my $r = shift;
	my @call = ();
	my $untaint;
	my %args = $r->args;
	foreach $untaint (keys %args) {
		if ($untaint =~ /^fi/) {
			if ($args{$untaint} =~ /^([-\@\w.]+)$/) {
				$args{$untaint} = $1;
			}
			else {
				$r->content_type('text/html');
				$r->send_http_header();
				print "TAINTING ATTEMPED INTERCEPTED AND LOGGED.\n";
				print "THANK YOU, DRIVE THROUGH.\n";
				return(OK);
			}
		}
	}
	my $count;
        $r->content_type('image/png');  
	$r->send_http_header();
	my $pee_eye_dee = $$;
	my $ahora = int(gettimeofday);
	my $tmpfilename = "/tmp/${pee_eye_dee}.${ahora}.png";
	push(@call,$tmpfilename);

	$count = 1;
        while ($args{"t$count"}) {
                $args{"c$count"} = "ff0000" unless $args{"c$count"};
		$count++;
        }
	$count = 1;
	while ($args{"t$count"}) {
		$args{"fi$count"} = "$args{\"t$count\"}.rrd" unless $args{"fi$count"};
		$count++;
	}
	$count = 1;
	while ($args{"t$count"}) {
		$args{"gt$count"} = "LINE2" unless $args{"gt$count"};
		$count++;
	}
	if ($args{'ti'}) {
		my $clean_title = uri_unescape($args{'ti'});
		push(@call,"--title","$clean_title");
	}
	push(@call,"-a","PNG");
	if ($args{'vl'}) {
		my $clean_vlabel = uri_unescape($args{'vl'});
		push(@call,"--vertical-label", "$clean_vlabel");
	}
	if ($args{'gw'}) {
		push(@call,"--width","$args{'gw'}");
	}
	if ($args{'gh'}) {
                push(@call,"--height","$args{'gh'}");
        }
	if ($args{'il'} eq "on") {
		push(@call,"--interlaced");
	}
        if ($args{'lg'} eq "on") {
                push(@call,"--logarithmic");
        }
	if ($args{'st'}) {
		push(@call,"--start",$args{'st'});
	} else {
		push(@call,"--start",'-86400');
	}
	if ($args{'et'}) {
		push(@call,"--end",$args{'et'});
	}
	$count = 1;
	while ($args{"t$count"}) {
		if ($args{"fi$count"} ne "COMPOSITE" ) {
			push(@call,"DEF:$args{\"t$count\"}=$RRD_BASE/$args{\"fi$count\"}:$args{\"t$count\"}:AVERAGE");
		}
		$count++;
	}
        $count = 1;
        while ($args{"cp$count"}) {
                my $clean_ct = uri_unescape($args{"ct$count"});
                my $clean_cp = uri_unescape($args{"cp$count"});
                $clean_cp =~ s/ /+/g;
                push(@call,"CDEF:$clean_ct\=$clean_cp");
                $count++;
        }
	$count = 1; 
	while ($args{"t$count"}) {
		if ($args{"gt$count"} ne "HIDDEN") {
			push(@call,"$args{\"gt$count\"}:$args{\"t$count\"}\#$args{\"c$count\"}:$args{\"tl$count\"}");
		}
		$count++;
	}
	if ($args{'lt0'}) {
		push(@call,"COMMENT:\\s");
		push(@call,"COMMENT:\\s");
	}
	$count = 0;
	while ($args{"lt$count"}) {
		push(@call,"GPRINT:$args{\"lt$count\"}:$args{\"lf$count\"}:$args{\"lx$count\"}");
		$count++;
	}
        my ($graphret,$xsize,$ysize) = RRDs::graph @call; 

	if ($args{'sx'} && $have_im) {
		my $x;
		my $image = new Image::Magick;
		$x = $image->Read($tmpfilename);
                # for debugging purposes...
		# my $xdebug=`echo "$x" > /tmp/debug.read` if "$x";
		$x = $image->Scale(geometry=>"$args{'sx'}x$args{'sy'}");
                # for debugging purposes...
                # my $xdebug=`echo "$x" > /tmp/debug.scale` if "$x"; 
		$image->Write($tmpfilename);
                # for debugging purposes...
                # my $xdebug=`echo "$x" > /tmp/debug.write` if "$x"; 
		# my $foo_debugg=`touch /tmp/called`;
		undef $image;
	}
	my $ERROR;
	if ($ERROR = RRDs::error) {
		my $dog=`echo "$args{"cp1"} $ERROR @call" > /tmp/rrd_ERROR_log`;
	} 
	else {
		my $fh = new FileHandle "<${tmpfilename}";
		$r->send_fd($fh);
		unlink(${tmpfilename});
		return(OK);
	}
}

# all modules succeed
1;

__END__
